@@ -1,4 +1,4 @@
-;;; misc.el --- basic editing commands for Emacs
+;;; misc.el --- some nonstandard basic editing commands for Emacs
 
 ;; Copyright (C) 1989 Free Software Foundation, Inc.
 
@@ -55,4 +55,6 @@ The characters copied are inserted in the buffer before point."
 				 (+ n (point)))))))
     (insert string)))
 
+(provide 'misc)
+
 ;;; misc.el ends here
