@@ -707,6 +707,31 @@ static tr_stack *mapping_stack_pointer;
   } while (0)
 
 
+/* Set C to the character code made from CHARSET and CODE.  This is
+   like MAKE_CHAR but check the validity of CHARSET and CODE.  If they
+   are not valid, set C to (CODE & 0xFF) because that is usually the
+   case that CCL_ReadMultibyteChar2 read an invalid code and it set
+   CODE to that invalid byte.  */
+
+#define CCL_MAKE_CHAR(charset, code, c)				\
+  do {								\
+    if (charset == CHARSET_ASCII)				\
+      c = code & 0xFF;						\
+    else if (CHARSET_DEFINED_P (charset)			\
+	     && (code & 0x7F) >= 32				\
+	     && (code < 256 || ((code >> 7) & 0x7F) >= 32))	\
+      {								\
+	int c1 = code & 0x7F, c2 = 0;				\
+								\
+	if (code >= 256)					\
+	  c2 = c1, c1 = (code >> 7) & 0x7F;			\
+	c = MAKE_NON_ASCII_CHAR (charset, c1, c2);		\
+      }								\
+    else							\
+      c = code & 0xFF;					\
+  } while (0)
+
+
 /* Execute CCL code on SRC_BYTES length text at SOURCE.  The resulting
    text goes to a place pointed by DESTINATION, the length of which
    should not exceed DST_BYTES.  The bytes actually processed is
@@ -902,7 +927,7 @@ ccl_driver (ccl, source, destination, src_bytes, dst_bytes, consumed)
 	  i = reg[RRR];
 	  j = XINT (ccl_prog[ic]);
 	  op = field1 >> 6;
-	  ic++;
+	  jump_address = ic + 1;
 	  goto ccl_set_expr;
 
 	case CCL_WriteRegister:	/* CCCCCCCCCCCCCCCCCCCrrrXXXXX */
@@ -922,6 +947,7 @@ ccl_driver (ccl, source, destination, src_bytes, dst_bytes, consumed)
 	  i = reg[RRR];
 	  j = reg[Rrr];
 	  op = field1 >> 6;
+	  jump_address = ic;
 	  goto ccl_set_expr;
 
 	case CCL_Call:		/* CCCCCCCCCCCCCCCCCCCC000XXXXX */
@@ -1087,6 +1113,7 @@ ccl_driver (ccl, source, destination, src_bytes, dst_bytes, consumed)
 	    {
 	      i = reg[rrr];
 	      CCL_WRITE_CHAR (i);
+	      ic = jump_address;
 	    }
 	  else if (!reg[rrr])
 	    ic = jump_address;
@@ -1228,20 +1255,12 @@ ccl_driver (ccl, source, destination, src_bytes, dst_bytes, consumed)
 
 	    case CCL_TranslateCharacter:
 	      i = reg[RRR]; /* charset */
-	      if (i == CHARSET_ASCII)
-		i = reg[rrr];
-	      else if (i == CHARSET_COMPOSITION)
+	      if (i == CHARSET_COMPOSITION)
 		{
 		  reg[RRR] = -1;
 		  break;
 		}
-	      else if (CHARSET_DIMENSION (i) == 1)
-		i = ((i - 0x70) << 7) | (reg[rrr] & 0x7F);
-	      else if (i < MIN_CHARSET_PRIVATE_DIMENSION2)
-		i = ((i - 0x8F) << 14) | (reg[rrr] & 0x3FFF);
-	      else
-		i = ((i - 0xE0) << 14) | (reg[rrr] & 0x3FFF);
-
+	      CCL_MAKE_CHAR (i, reg[rrr], i);
 	      op = translate_char (GET_TRANSLATION_TABLE (reg[Rrr]),
 				   i, -1, 0, 0);
 	      SPLIT_CHAR (op, reg[RRR], i, j);
@@ -1255,20 +1274,12 @@ ccl_driver (ccl, source, destination, src_bytes, dst_bytes, consumed)
 	      op = XINT (ccl_prog[ic]); /* table */
 	      ic++;
 	      i = reg[RRR]; /* charset */
-	      if (i == CHARSET_ASCII)
-		i = reg[rrr];
-	      else if (i == CHARSET_COMPOSITION)
+	      if (i == CHARSET_COMPOSITION)
 		{
 		  reg[RRR] = -1;
 		  break;
 		}
-	      else if (CHARSET_DIMENSION (i) == 1)
-		i = ((i - 0x70) << 7) | (reg[rrr] & 0x7F);
-	      else if (i < MIN_CHARSET_PRIVATE_DIMENSION2)
-		i = ((i - 0x8F) << 14) | (reg[rrr] & 0x3FFF);
-	      else
-		i = ((i - 0xE0) << 14) | (reg[rrr] & 0x3FFF);
-
+	      CCL_MAKE_CHAR (i, reg[rrr], i);
 	      op = translate_char (GET_TRANSLATION_TABLE (op), i, -1, 0, 0);
 	      SPLIT_CHAR (op, reg[RRR], i, j);
 	      if (j != -1)
@@ -1475,6 +1486,7 @@ ccl_driver (ccl, source, destination, src_bytes, dst_bytes, consumed)
 		      }
 		    else if (EQ (content, Qlambda))
 		      {
+			reg[RRR] = i;
 			break;
 		      }
 		    else
@@ -1517,13 +1529,13 @@ ccl_driver (ccl, source, destination, src_bytes, dst_bytes, consumed)
 		  reg[RRR] = -1;
 		else
 		  {
+		    reg[RRR] = 0;
 		    content = XVECTOR (map)->contents[point];
 		    if (NILP (content))
 		      reg[RRR] = -1;
 		    else if (NUMBERP (content))
 		      reg[rrr] = XINT (content);
-		    else if (EQ (content, Qt))
-		      reg[RRR] = i;
+		    else if (EQ (content, Qt));
 		    else if (CONSP (content))
 		      {
 			attrib = XCONS (content)->car;
