@@ -1,5 +1,5 @@
 /* Generic frame functions.
-   Copyright (C) 1993, 1994, 1995 Free Software Foundation.
+   Copyright (C) 1993, 1994, 1995, 1997 Free Software Foundation.
 
 This file is part of GNU Emacs.
 
@@ -21,9 +21,20 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #include "lisp.h"
+#include "charset.h"
+#ifdef HAVE_X_WINDOWS
+#include "xterm.h"
+#endif
 #include "frame.h"
+#ifdef HAVE_WINDOW_SYSTEM
+#include "fontset.h"
+#endif
 #include "termhooks.h"
+#include "dispextern.h"
 #include "window.h"
 #ifdef MSDOS
 #include "msdos.h"
@@ -62,8 +73,6 @@ Boston, MA 02111-1307, USA.  */
       (setq symbol-list (cdr symbol-list)))))
   */        
 
-/* We need most of these symbols even if not MULTI_FRAME;
-   easiest to define them all, all of the time.  */
 /*&&& symbols declared here &&&*/
 Lisp_Object Qframep;
 Lisp_Object Qframe_live_p;
@@ -77,16 +86,16 @@ Lisp_Object Qunsplittable;
 Lisp_Object Qmenu_bar_lines;
 Lisp_Object Qwidth;
 Lisp_Object Qx;
-Lisp_Object Qwin32;
+Lisp_Object Qw32;
+Lisp_Object Qpc;
 Lisp_Object Qvisible;
 Lisp_Object Qbuffer_predicate;
+Lisp_Object Qbuffer_list;
 Lisp_Object Qtitle;
 
 Lisp_Object Vterminal_frame;
 Lisp_Object Vdefault_frame_alist;
 
-Lisp_Object Qmouse_leave_buffer_hook;
-
 static void
 syms_of_frame_1 ()
 {
@@ -115,18 +124,19 @@ syms_of_frame_1 ()
   staticpro (&Qwidth);
   Qx = intern ("x");
   staticpro (&Qx);
-  Qwin32 = intern ("win32");
-  staticpro (&Qwin32);
+  Qw32 = intern ("w32");
+  staticpro (&Qw32);
+  Qpc = intern ("pc");
+  staticpro (&Qpc);
   Qvisible = intern ("visible");
   staticpro (&Qvisible);
   Qbuffer_predicate = intern ("buffer-predicate");
   staticpro (&Qbuffer_predicate);
+  Qbuffer_list = intern ("buffer-list");
+  staticpro (&Qbuffer_list);
   Qtitle = intern ("title");
   staticpro (&Qtitle);
 
-  Qmouse_leave_buffer_hook = intern ("mouse-leave-buffer-hook");
-  staticpro (&Qmouse_leave_buffer_hook);
-
   DEFVAR_LISP ("default-frame-alist", &Vdefault_frame_alist,
     "Alist of default values for frame creation.\n\
 These may be set in your init file, like this:\n\
@@ -164,7 +174,7 @@ set_menu_bar_lines_1 (window, n)
     }
 }
 
-static void
+void
 set_menu_bar_lines (f, value, oldval)
      struct frame *f;
      Lisp_Object value, oldval;
@@ -174,7 +184,7 @@ set_menu_bar_lines (f, value, oldval)
 
   /* Right now, menu bars don't work properly in minibuf-only frames;
      most of the commands try to apply themselves to the minibuffer
-     frame itslef, and get an error because you can't switch buffers
+     frame itself, and get an error because you can't switch buffers
      in or split the minibuffer window.  */
   if (FRAME_MINIBUF_ONLY_P (f))
     return;
@@ -193,8 +203,6 @@ set_menu_bar_lines (f, value, oldval)
     }
 }
 
-#ifdef MULTI_FRAME
-
 #include "buffer.h"
 
 /* These help us bind and responding to switch-frame events.  */
@@ -227,9 +235,10 @@ See also `frame-live-p'.")
       return Qt;
     case output_x_window:
       return Qx;
-    case output_win32:
-      return Qwin32;
-      /* The `pc' case is in the Fframep below.  */
+    case output_w32:
+      return Qw32;
+    case output_msdos_raw:
+      return Qpc;
     default:
       abort ();
     }
@@ -286,7 +295,7 @@ make_frame (mini_p)
   f->focus_frame = Qnil;
   f->explicit_name = 0;
   f->can_have_scroll_bars = 0;
-  f->has_vertical_scroll_bars = 0;
+  f->vertical_scroll_bar_type = vertical_scroll_bar_none;
   f->param_alist = Qnil;
   f->scroll_bars = Qnil;
   f->condemned_scroll_bars = Qnil;
@@ -295,6 +304,7 @@ make_frame (mini_p)
   f->menu_bar_vector = Qnil;
   f->menu_bar_items_used = 0;
   f->buffer_predicate = Qnil;
+  f->buffer_list = Qnil;
 #ifdef MULTI_KBOARD
   f->kboard = initial_kboard;
 #endif
@@ -324,7 +334,7 @@ make_frame (mini_p)
      just so that there is "something there."
      Correct size will be set up later with change_frame_size.  */
 
-  f->width = 10;
+  SET_FRAME_WIDTH (f, 10);
   f->height = 10;
 
   XSETFASTINT (XWINDOW (root_window)->width, 10);
@@ -346,8 +356,10 @@ make_frame (mini_p)
     /* If buf is a 'hidden' buffer (i.e. one whose name starts with
        a space), try to find another one.  */
     if (XSTRING (Fbuffer_name (buf))->data[0] == ' ')
-      buf = Fother_buffer (buf, Qnil);
+      buf = Fother_buffer (buf, Qnil, Qnil);
     Fset_window_buffer (root_window, buf);
+
+    f->buffer_list = Fcons (buf, Qnil);
   }
 
   if (mini_p)
@@ -365,9 +377,14 @@ make_frame (mini_p)
      a newly-created, never-selected window.  */
   XSETFASTINT (XWINDOW (f->selected_window)->use_time, ++window_select_count);
 
+#ifdef HAVE_WINDOW_SYSTEM
+  f->fontset_data = alloc_fontset_data ();
+#endif
+
   return f;
 }
 
+#ifdef HAVE_WINDOW_SYSTEM
 /* Make a frame using a separate minibuffer window on another frame.
    MINI_WINDOW is the minibuffer window to use.  nil means use the
    default (the global minibuffer).  */
@@ -379,6 +396,7 @@ make_frame_without_minibuffer (mini_window, kb, display)
      Lisp_Object display;
 {
   register struct frame *f;
+  struct gcpro gcpro1;
 
   if (!NILP (mini_window))
     CHECK_LIVE_WINDOW (mini_window, 0);
@@ -398,10 +416,16 @@ make_frame_without_minibuffer (mini_window, kb, display)
       if (!FRAMEP (kb->Vdefault_minibuffer_frame)
 	  || ! FRAME_LIVE_P (XFRAME (kb->Vdefault_minibuffer_frame)))
 	{
+          Lisp_Object frame_dummy;
+
+          XSETFRAME (frame_dummy, f);
+          GCPRO1 (frame_dummy);
 	  /* If there's no minibuffer frame to use, create one.  */
-	  kb->Vdefault_minibuffer_frame
-	    = call1 (intern ("make-initial-minibuffer-frame"), display);
+	  kb->Vdefault_minibuffer_frame =
+	    call1 (intern ("make-initial-minibuffer-frame"), display);
+          UNGCPRO;
 	}
+   
       mini_window = XFRAME (kb->Vdefault_minibuffer_frame)->minibuffer_window;
     }
 
@@ -454,6 +478,7 @@ make_minibuffer_frame ()
 		       : Fcar (Vminibuffer_list)));
   return f;
 }
+#endif /* HAVE_WINDOW_SYSTEM */
 
 /* Construct a frame that refers to the terminal (stdin and stdout).  */
 
@@ -486,19 +511,18 @@ make_terminal_frame ()
   Vframe_list = Fcons (frame, Vframe_list);
 
   terminal_frame_count++;
-  if (terminal_frame_count == 1)
-    {
-      f->name = build_string ("Emacs");
-    }
-  else
-    {
-      sprintf (name, "Emacs-%d", terminal_frame_count);
-      f->name = build_string (name);
-    }
+  sprintf (name, "F%d", terminal_frame_count);
+  f->name = build_string (name);
 
   f->visible = 1;		/* FRAME_SET_VISIBLE wd set frame_garbaged. */
   f->async_visible = 1;		/* Don't let visible be cleared later. */
+#ifdef MSDOS
+  f->output_data.x = &the_only_x_display;
+  f->output_method = output_msdos_raw;
+  init_frame_faces (f);
+#else /* not MSDOS */
   f->output_data.nothing = 1;	/* Nonzero means frame isn't deleted.  */
+#endif
   return f;
 }
 
@@ -515,8 +539,13 @@ Note that changing the size of one terminal frame automatically affects all.")
   struct frame *f;
   Lisp_Object frame;
 
+#ifdef MSDOS
+  if (selected_frame->output_method != output_msdos_raw)
+    abort ();
+#else
   if (selected_frame->output_method != output_termcap)
     error ("Not using an ASCII terminal now; cannot make a new ASCII frame");
+#endif
 
   f = make_terminal_frame ();
   change_frame_size (f, FRAME_HEIGHT (selected_frame),
@@ -524,6 +553,7 @@ Note that changing the size of one terminal frame automatically affects all.")
   remake_frame_glyphs (f);
   calculate_costs (f);
   XSETFRAME (frame, f);
+  Fmodify_frame_parameters (frame, Vdefault_frame_alist);
   Fmodify_frame_parameters (frame, parms);
   f->face_alist = selected_frame->face_alist;
   return frame;
@@ -1050,6 +1080,13 @@ but if the second optional argument FORCE is non-nil, you may do so.")
   if (NILP (force) && !other_visible_frames (f))
     error ("Attempt to delete the sole visible or iconified frame");
 
+#if 0
+  /* This is a nice idea, but x_connection_closed needs to be able
+     to delete the last frame, if it is gone.  */
+  if (NILP (XCONS (Vframe_list)->cdr))
+    error ("Attempt to delete the only frame");
+#endif
+
   /* Does this frame have a minibuffer, and is it the surrogate
      minibuffer for any other frame?  */
   if (FRAME_HAS_MINIBUF_P (XFRAME (frame)))
@@ -1121,6 +1158,12 @@ but if the second optional argument FORCE is non-nil, you may do so.")
   Vframe_list = Fdelq (frame, Vframe_list);
   FRAME_SET_VISIBLE (f, 0);
 
+  if (echo_area_glyphs == FRAME_MESSAGE_BUF (f))
+    {
+      echo_area_glyphs = 0;
+      previous_echo_glyphs = 0;
+    }
+
   if (f->namebuf)
     free (f->namebuf);
   if (FRAME_CURRENT_GLYPHS (f))
@@ -1137,6 +1180,13 @@ but if the second optional argument FORCE is non-nil, you may do so.")
     free (FRAME_INSERTN_COST (f));
   if (FRAME_DELETE_COST (f))
     free (FRAME_DELETE_COST (f));
+  if (FRAME_MESSAGE_BUF (f))
+    free (FRAME_MESSAGE_BUF (f));
+
+#ifdef HAVE_WINDOW_SYSTEM
+  /* Free all fontset data.  */
+  free_fontset_data (FRAME_FONTSET_DATA (f));
+#endif
 
   /* Since some events are handled at the interrupt level, we may get
      an event for f at any time; if we zero out the frame's display
@@ -1232,6 +1282,9 @@ but if the second optional argument FORCE is non-nil, you may do so.")
 	FRAME_KBOARD (f)->Vdefault_minibuffer_frame = Qnil;
     }
 
+  /* Cause frame titles to update--necessary if we now have just one frame.  */
+  update_mode_lines = 1;
+
   return Qnil;
 }
 
@@ -1259,7 +1312,7 @@ and nil for X and Y.")
 #ifdef HAVE_MOUSE
   /* It's okay for the hook to refrain from storing anything.  */
   if (mouse_position_hook)
-    (*mouse_position_hook) (&f, 0,
+    (*mouse_position_hook) (&f, -1,
 			    &lispy_dummy, &party_dummy,
 			    &x, &y,
 			    &long_dummy);
@@ -1299,7 +1352,7 @@ and nil for X and Y.")
 #ifdef HAVE_MOUSE
   /* It's okay for the hook to refrain from storing anything.  */
   if (mouse_position_hook)
-    (*mouse_position_hook) (&f, 0,
+    (*mouse_position_hook) (&f, -1,
 			    &lispy_dummy, &party_dummy,
 			    &x, &y,
 			    &long_dummy);
@@ -1310,7 +1363,11 @@ and nil for X and Y.")
 
 DEFUN ("set-mouse-position", Fset_mouse_position, Sset_mouse_position, 3, 3, 0,
   "Move the mouse pointer to the center of character cell (X,Y) in FRAME.\n\
-Note, this is a no-op for an X frame that is not visible.\n\
+Coordinates are relative to the frame, not a window,\n\
+so the coordinates of the top left character in the frame\n\
+may be nonzero due to left-hand scroll bars or the menu bar.\n\
+\n\
+This function is a no-op for an X frame that is not visible.\n\
 If you have just created a frame, you must wait for it to become visible\n\
 before calling this function on it, like this.\n\
   (while (not (frame-visible-p frame)) (sleep-for .5))")
@@ -1325,7 +1382,15 @@ before calling this function on it, like this.\n\
 #ifdef HAVE_WINDOW_SYSTEM
   if (FRAME_WINDOW_P (XFRAME (frame)))
     /* Warping the mouse will cause  enternotify and focus events. */
-    x_set_mouse_position (XFRAME (frame), x, y);
+    x_set_mouse_position (XFRAME (frame), XINT (x), XINT (y));
+#else
+#if defined (MSDOS) && defined (HAVE_MOUSE)
+  if (FRAME_MSDOS_P (XFRAME (frame)))
+    {
+      Fselect_frame (frame, Qnil);
+      mouse_moveto (XINT (x), XINT (y));
+    }
+#endif
 #endif
 
   return Qnil;
@@ -1349,12 +1414,22 @@ before calling this function on it, like this.\n\
 #ifdef HAVE_WINDOW_SYSTEM
   if (FRAME_WINDOW_P (XFRAME (frame)))
     /* Warping the mouse will cause  enternotify and focus events. */
-    x_set_mouse_pixel_position (XFRAME (frame), x, y);
+    x_set_mouse_pixel_position (XFRAME (frame), XINT (x), XINT (y));
+#else
+#if defined (MSDOS) && defined (HAVE_MOUSE)
+  if (FRAME_MSDOS_P (XFRAME (frame)))
+    {
+      Fselect_frame (frame, Qnil);
+      mouse_moveto (XINT (x), XINT (y));
+    }
+#endif
 #endif
 
   return Qnil;
 }
 
+static void make_frame_visible_1 P_ ((Lisp_Object));
+
 DEFUN ("make-frame-visible", Fmake_frame_visible, Smake_frame_visible,
        0, 1, "",
   "Make the frame FRAME visible (assuming it is an X-window).\n\
@@ -1376,12 +1451,37 @@ If omitted, FRAME defaults to the currently selected frame.")
     }
 #endif
 
+  make_frame_visible_1 (XFRAME (frame)->root_window);
+
   /* Make menu bar update for the Buffers and Frams menus.  */
   windows_or_buffers_changed++;
 
   return frame;
 }
 
+/* Update the display_time slot of the buffers shown in WINDOW
+   and all its descendents.  */
+
+static void
+make_frame_visible_1 (window)
+     Lisp_Object window;
+{
+  struct window *w;
+
+  for (;!NILP (window); window = w->next)
+    {
+      w = XWINDOW (window);
+
+      if (!NILP (w->buffer))
+	XBUFFER (w->buffer)->display_time = Fcurrent_time ();
+
+      if (!NILP (w->vchild))
+	make_frame_visible_1 (w->vchild);
+      if (!NILP (w->hchild))
+	make_frame_visible_1 (w->hchild);
+    }
+}
+
 DEFUN ("make-frame-invisible", Fmake_frame_invisible, Smake_frame_invisible,
        0, 2, "",
   "Make the frame FRAME invisible (assuming it is an X-window).\n\
@@ -1588,13 +1688,6 @@ The redirection lasts until `redirect-frame-focus' is called to change it.")
 
   XFRAME (frame)->focus_frame = focus_frame;
 
-  /* I think this should be done with a hook.  */
-#ifdef HAVE_WINDOW_SYSTEM
-  if (!NILP (focus_frame) && ! EQ (focus_frame, frame)
-      && (FRAME_WINDOW_P (XFRAME (focus_frame))))
-    Ffocus_frame (focus_frame);
-#endif
-
   if (frame_rehighlight_hook)
     (*frame_rehighlight_hook) (XFRAME (frame));
   
@@ -1634,9 +1727,63 @@ get_frame_param (frame, prop)
 /* Return the buffer-predicate of the selected frame.  */
 
 Lisp_Object
-frame_buffer_predicate ()
+frame_buffer_predicate (frame)
+     Lisp_Object frame;
 {
-  return selected_frame->buffer_predicate;
+  return XFRAME (frame)->buffer_predicate;
+}
+
+/* Return the buffer-list of the selected frame.  */
+
+Lisp_Object
+frame_buffer_list (frame)
+     Lisp_Object frame;
+{
+  return XFRAME (frame)->buffer_list;
+}
+
+/* Set the buffer-list of the selected frame.  */
+
+void
+set_frame_buffer_list (frame, list)
+     Lisp_Object frame, list;
+{
+  XFRAME (frame)->buffer_list = list;
+}
+
+/* Discard BUFFER from the buffer-list of each frame.  */
+
+void
+frames_discard_buffer (buffer)
+     Lisp_Object buffer;
+{
+  Lisp_Object frame, tail;
+
+  FOR_EACH_FRAME (tail, frame)
+    {
+      XFRAME (frame)->buffer_list
+	= Fdelq (buffer, XFRAME (frame)->buffer_list);
+    }
+}
+
+/* Move BUFFER to the end of the buffer-list of each frame.  */
+
+void
+frames_bury_buffer (buffer)
+     Lisp_Object buffer;
+{
+  Lisp_Object frame, tail;
+
+  FOR_EACH_FRAME (tail, frame)
+    {
+      struct frame *f = XFRAME (frame);
+      Lisp_Object found;
+      
+      found = Fmemq (buffer, f->buffer_list);
+      if (!NILP (found))
+	f->buffer_list = nconc2 (Fdelq (buffer, f->buffer_list),
+				 Fcons (buffer, Qnil));
+    }
 }
 
 /* Modify the alist in *ALISTPTR to associate PROP with VAL.
@@ -1656,6 +1803,65 @@ store_in_alist (alistptr, prop, val)
     Fsetcdr (tem, val);
 }
 
+static int
+frame_name_fnn_p (str, len)
+     char *str;
+     int len;
+{
+  if (len > 1 && str[0] == 'F')
+    {
+      char *end_ptr;
+      long num = strtol (str + 1, &end_ptr, 10);
+
+      if (end_ptr == str + len)
+	return 1;
+    }
+  return 0;
+}
+
+/* Set the name of the terminal frame.  Also used by MSDOS frames.
+   Modeled after x_set_name which is used for WINDOW frames.  */
+
+void
+set_term_frame_name (f, name)
+     struct frame *f;
+     Lisp_Object name;
+{
+  f->explicit_name = ! NILP (name);
+
+  /* If NAME is nil, set the name to F<num>.  */
+  if (NILP (name))
+    {
+      char namebuf[20];
+
+      /* Check for no change needed in this very common case
+	 before we do any consing.  */
+      if (frame_name_fnn_p (XSTRING (f->name)->data,
+			    STRING_BYTES (XSTRING (f->name))))
+	return;
+
+      terminal_frame_count++;
+      sprintf (namebuf, "F%d", terminal_frame_count);
+      name = build_string (namebuf);
+    }
+  else
+    {
+      CHECK_STRING (name, 0);
+
+      /* Don't change the name if it's already NAME.  */
+      if (! NILP (Fstring_equal (name, f->name)))
+	return;
+
+      /* Don't allow the user to set the frame name to F<num>, so it
+	 doesn't clash with the names we generate for terminal frames.  */
+      if (frame_name_fnn_p (XSTRING (name)->data, STRING_BYTES (XSTRING (name))))
+	error ("Frame names of the form F<num> are usurped by Emacs");
+    }
+
+  f->name = name;
+  update_mode_lines = 1;
+}
+
 void
 store_frame_param (f, prop, val)
      struct frame *f;
@@ -1663,6 +1869,12 @@ store_frame_param (f, prop, val)
 {
   register Lisp_Object tem;
 
+  if (EQ (prop, Qbuffer_list))
+    {
+      f->buffer_list = val;
+      return;
+    }
+
   tem = Fassq (prop, f->param_alist);
   if (EQ (tem, Qnil))
     f->param_alist = Fcons (Fcons (prop, val), f->param_alist);
@@ -1673,16 +1885,21 @@ store_frame_param (f, prop, val)
     f->buffer_predicate = val;
 
   if (! FRAME_WINDOW_P (f))
-    if (EQ (prop, Qmenu_bar_lines))
-      set_menu_bar_lines (f, val, make_number (FRAME_MENU_BAR_LINES (f)));
+    {
+      if (EQ (prop, Qmenu_bar_lines))
+	set_menu_bar_lines (f, val, make_number (FRAME_MENU_BAR_LINES (f)));
+      else if (EQ (prop, Qname))
+	set_term_frame_name (f, val);
+    }
 
   if (EQ (prop, Qminibuffer) && WINDOWP (val))
     {
       if (! MINI_WINDOW_P (XWINDOW (val)))
 	error ("Surrogate minibuffer windows must be minibuffer windows.");
 
-      if (FRAME_HAS_MINIBUF_P (f) || FRAME_MINIBUF_ONLY_P (f))
-	error ("can't change the surrogate minibuffer of a frame with its own minibuffer");
+      if (FRAME_HAS_MINIBUF_P (f) || FRAME_MINIBUF_ONLY_P (f)
+	  && !EQ (val, f->minibuffer_window))
+	error ("Can't change the surrogate minibuffer of a frame with its own minibuffer");
 
       /* Install the chosen minibuffer window, with proper buffer.  */
       f->minibuffer_window = val;
@@ -1713,6 +1930,22 @@ If FRAME is omitted, return information on the currently selected frame.")
     return Qnil;
 
   alist = Fcopy_alist (f->param_alist);
+#ifdef MSDOS
+  if (FRAME_MSDOS_P (f))
+    {
+      static char *colornames[16] = 
+	{
+	  "black", "blue", "green", "cyan", "red", "magenta", "brown",
+	  "lightgray", "darkgray", "lightblue", "lightgreen", "lightcyan",
+	  "lightred", "lightmagenta", "yellow", "white"
+	};
+      store_in_alist (&alist, intern ("foreground-color"),
+		      build_string (colornames[FRAME_FOREGROUND_PIXEL (f)]));
+      store_in_alist (&alist, intern ("background-color"),
+		      build_string (colornames[FRAME_BACKGROUND_PIXEL (f)]));
+    }
+  store_in_alist (&alist, intern ("font"), build_string ("default"));
+#endif
   store_in_alist (&alist, Qname, f->name);
   height = (FRAME_NEW_HEIGHT (f) ? FRAME_NEW_HEIGHT (f) : FRAME_HEIGHT (f));
   store_in_alist (&alist, Qheight, make_number (height));
@@ -1724,6 +1957,8 @@ If FRAME is omitted, return information on the currently selected frame.")
 		   : FRAME_MINIBUF_ONLY_P (f) ? Qonly
 		   : FRAME_MINIBUF_WINDOW (f)));
   store_in_alist (&alist, Qunsplittable, (FRAME_NO_SPLIT_P (f) ? Qt : Qnil));
+  store_in_alist (&alist, Qbuffer_list,
+		  frame_buffer_list (Fselected_frame ()));
 
   /* I think this should be done with a hook.  */
 #ifdef HAVE_WINDOW_SYSTEM
@@ -1745,7 +1980,9 @@ DEFUN ("modify-frame-parameters", Fmodify_frame_parameters,
   "Modify the parameters of frame FRAME according to ALIST.\n\
 ALIST is an alist of parameters to change and their new values.\n\
 Each element of ALIST has the form (PARM . VALUE), where PARM is a symbol.\n\
-The meaningful PARMs depend on the kind of frame; undefined PARMs are ignored.")
+The meaningful PARMs depend on the kind of frame.\n\
+Undefined PARMs are ignored, but stored in the frame's parameter list\n\
+so that `frame-parameters' will return them.")
   (frame, alist)
      Lisp_Object frame, alist;
 {
@@ -1766,13 +2003,40 @@ The meaningful PARMs depend on the kind of frame; undefined PARMs are ignored.")
     x_set_frame_parameters (f, alist);
   else
 #endif
-    for (tail = alist; !EQ (tail, Qnil); tail = Fcdr (tail))
-      {
-	elt = Fcar (tail);
-	prop = Fcar (elt);
-	val = Fcdr (elt);
-	store_frame_param (f, prop, val);
-      }
+#ifdef MSDOS
+  if (FRAME_MSDOS_P (f))
+    IT_set_frame_parameters (f, alist);
+  else
+#endif
+    {
+      int length = XINT (Flength (alist));
+      int i;
+      Lisp_Object *parms
+	= (Lisp_Object *) alloca (length * sizeof (Lisp_Object));
+      Lisp_Object *values
+	= (Lisp_Object *) alloca (length * sizeof (Lisp_Object));
+
+      /* Extract parm names and values into those vectors.  */
+
+      i = 0;
+      for (tail = alist; CONSP (tail); tail = Fcdr (tail))
+	{
+	  Lisp_Object elt, prop, val;
+
+	  elt = Fcar (tail);
+	  parms[i] = Fcar (elt);
+	  values[i] = Fcdr (elt);
+	  i++;
+	}
+
+      /* Now process them in reverse of specified order.  */
+      for (i--; i >= 0; i--)
+	{
+	  prop = parms[i];
+	  val = values[i];
+	  store_frame_param (f, prop, val);
+	}
+    }
 
   return Qnil;
 }
@@ -1835,6 +2099,8 @@ For a terminal screen, the value is always 1.")
 DEFUN ("frame-pixel-height", Fframe_pixel_height, 
        Sframe_pixel_height, 0, 1, 0,
   "Return a FRAME's height in pixels.\n\
+This counts only the height available for text lines,\n\
+not menu bars on window-system Emacs frames.\n\
 For a terminal frame, the result really gives the height in characters.\n\
 If FRAME is omitted, the selected frame is used.")
   (frame)
@@ -2000,6 +2266,7 @@ the rightmost or bottommost possible position (that stays within the screen).")
 }
 
 
+void
 syms_of_frame ()
 {
   syms_of_frame_1 ();
@@ -2072,6 +2339,7 @@ displayed.");
   defsubr (&Sset_frame_position);
 }
 
+void
 keys_of_frame ()
 {
   initial_define_lispy_key (global_map, "switch-frame", "handle-switch-frame");
@@ -2079,427 +2347,3 @@ keys_of_frame ()
   initial_define_lispy_key (global_map, "iconify-frame", "ignore-event");
   initial_define_lispy_key (global_map, "make-frame-visible", "ignore-event");
 }
-
-#else /* not MULTI_FRAME */
-
-/* If we're not using multi-frame stuff, we still need to provide some
-   support functions.  */
-
-/* Unless this function is defined, providing set-frame-height and
-   set-frame-width doesn't help compatibility any, since they both
-   want this as their first argument.  */
-DEFUN ("selected-frame", Fselected_frame, Sselected_frame, 0, 0, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  ()
-{
-  /* For your possible information, this code is unfolded into the
-     second WINDOW_FRAME in frame.h.  */     
-  Lisp_Object tem;
-  XSETFASTINT (tem, 0);
-  return tem;
-}
-
-DEFUN ("active-minibuffer-window", Factive_minibuffer_window,
-       Sactive_minibuffer_window, 0, 0, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  ()
-{
-  return minibuf_level ? minibuf_window : Qnil;
-}
-
-DEFUN ("window-frame", Fwindow_frame, Swindow_frame, 1, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (window)
-     Lisp_Object window;
-{
-  /* For your possible information, this code is unfolded into the
-     second WINDOW_FRAME in frame.h.  */     
-  Lisp_Object tem;
-  XSETFASTINT (tem, 0);
-  return tem;
-}
-
-DEFUN ("frame-first-window", Fframe_first_window, Sframe_first_window, 0, 1, 0,
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  Lisp_Object w;
-
-  w = FRAME_ROOT_WINDOW (selected_frame);
-
-  while (NILP (XWINDOW (w)->buffer))
-    {
-      if (! NILP (XWINDOW (w)->hchild))
-	w = XWINDOW (w)->hchild;
-      else if (! NILP (XWINDOW (w)->vchild))
-	w = XWINDOW (w)->vchild;
-      else
-	abort ();
-    }
-  return w;
-}
-
-DEFUN ("framep", Fframep, Sframep, 1, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (object)
-     Lisp_Object object;
-{
-#ifdef MSDOS
-  if (FRAME_X_P (object))
-    return intern ("pc");
-#endif
-  return Qnil;
-}
-
-DEFUN ("set-frame-height", Fset_frame_height, Sset_frame_height, 2, 3, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame, rows, pretend)
-     Lisp_Object frame, rows, pretend;
-{
-  CHECK_NUMBER (rows, 0);
-
-  change_frame_size (0, XINT (rows), 0, !NILP (pretend), 0);
-  return Qnil;
-}
-
-DEFUN ("set-frame-width", Fset_frame_width, Sset_frame_width, 2, 3, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame, cols, pretend)
-     Lisp_Object frame, cols, pretend;
-{
-  CHECK_NUMBER (cols, 0);
-
-  change_frame_size (0, 0, XINT (cols), !NILP (pretend), 0);
-  return Qnil;
-}
-
-DEFUN ("set-frame-size", Fset_frame_size, Sset_frame_size, 3, 3, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame, cols, rows)
-     Lisp_Object frame, cols, rows;
-{
-  CHECK_NUMBER (cols, 2);
-  CHECK_NUMBER (rows, 1);
-
-  change_frame_size (0, XINT (rows), XINT (cols), 0, 0);
-
-  return Qnil;
-}
-
-DEFUN ("frame-height", Fframe_height, Sframe_height, 0, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-    Lisp_Object frame;
-{
-  return make_number (FRAME_HEIGHT (selected_frame));
-}
-
-DEFUN ("frame-width", Fframe_width, Sframe_width, 0, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-    Lisp_Object frame;
-{
-  return make_number (FRAME_WIDTH (selected_frame));
-}
-
-DEFUN ("frame-char-height", Fframe_char_height, Sframe_char_height,
-  0, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  return make_number (1);
-}
-
-
-DEFUN ("frame-char-width", Fframe_char_width, Sframe_char_width,
-  0, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  return make_number (1);
-}
-
-DEFUN ("frame-pixel-height", Fframe_pixel_height, 
-       Sframe_pixel_height, 0, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  return make_number (FRAME_HEIGHT (f));
-}
-
-DEFUN ("frame-pixel-width", Fframe_pixel_width, 
-       Sframe_pixel_width, 0, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  return make_number (FRAME_WIDTH (f));
-}
-
-/* These are for backward compatibility with Emacs 18.  */
-
-DEFUN ("set-screen-height", Fset_screen_height, Sset_screen_height, 1, 2, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (lines, pretend)
-     Lisp_Object lines, pretend;
-{
-  CHECK_NUMBER (lines, 0);
-
-  change_frame_size (0, XINT (lines), 0, !NILP (pretend), 0);
-  return Qnil;
-}
-
-DEFUN ("set-screen-width", Fset_screen_width, Sset_screen_width, 1, 2, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (cols, pretend)
-     Lisp_Object cols, pretend;
-{
-  CHECK_NUMBER (cols, 0);
-
-  change_frame_size (0, 0, XINT (cols), !NILP (pretend), 0);
-  return Qnil;
-}
-
-DEFUN ("mouse-position", Fmouse_position, Smouse_position, 0, 0, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  ()
-{
-#ifdef HAVE_MOUSE
-  if (mouse_position_hook)
-    {
-      FRAME_PTR f;
-      Lisp_Object lispy_dummy;
-      enum scroll_bar_part party_dummy;
-      Lisp_Object x, y;
-      unsigned long long_dummy;
-
-      (*mouse_position_hook) (&f, 0,
-			      &lispy_dummy, &party_dummy,
-			      &x, &y,
-			      &long_dummy);      
-      return Fcons (Fselected_frame (), Fcons (x, y));
-    }
-#endif
-  return Fcons (Qnil, Fcons (Qnil, Qnil));
-}
-
-void
-store_in_alist (alistptr, prop, val)
-     Lisp_Object *alistptr, val;
-     Lisp_Object prop;
-{
-  register Lisp_Object tem;
-
-  tem = Fassq (prop, *alistptr);
-  if (EQ (tem, Qnil))
-    *alistptr = Fcons (Fcons (prop, val), *alistptr);
-  else
-    Fsetcdr (tem, val);
-}
-
-DEFUN ("frame-parameters", Fframe_parameters, Sframe_parameters, 0, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  Lisp_Object alist;
-  FRAME_PTR f;
-  int height, width;
-
-  if (EQ (frame, Qnil))
-    f = selected_frame;
-  else
-    {
-      CHECK_FRAME (frame, 0);
-      f = XFRAME (frame);
-    }
-
-  if (!FRAME_LIVE_P (f))
-    return Qnil;
-
-  alist = Qnil;
-#ifdef MSDOS
-  if (FRAME_X_P (f))
-    {
-      static char *colornames[16] = 
-	{
-	  "black", "blue", "green", "cyan", "red", "magenta", "brown",
-	  "lightgray", "darkgray", "lightblue", "lightgreen", "lightcyan",
-	  "lightred", "lightmagenta", "yellow", "white"
-	};
-      store_in_alist (&alist, intern ("foreground-color"),
-		      build_string (colornames[FRAME_FOREGROUND_PIXEL (f)]));
-      store_in_alist (&alist, intern ("background-color"),
-		      build_string (colornames[FRAME_BACKGROUND_PIXEL (f)]));
-    }
-#endif
-  store_in_alist (&alist, intern ("font"), build_string ("default"));
-  store_in_alist (&alist, Qname, build_string ("emacs"));
-  height = (FRAME_NEW_HEIGHT (f) ? FRAME_NEW_HEIGHT (f) : FRAME_HEIGHT (f));
-  store_in_alist (&alist, Qheight, make_number (height));
-  width = (FRAME_NEW_WIDTH (f) ? FRAME_NEW_WIDTH (f) : FRAME_WIDTH (f));
-  store_in_alist (&alist, Qwidth, make_number (width));
-  store_in_alist (&alist, Qmodeline, (FRAME_WANTS_MODELINE_P (f) ? Qt : Qnil));
-  store_in_alist (&alist, Qminibuffer, FRAME_MINIBUF_WINDOW (f));
-  store_in_alist (&alist, Qunsplittable, (FRAME_NO_SPLIT_P (f) ? Qt : Qnil));
-  store_in_alist (&alist, Qmenu_bar_lines, (FRAME_MENU_BAR_LINES (f)));
-
-  return alist;
-}
-
-DEFUN ("modify-frame-parameters", Fmodify_frame_parameters, 
-       Smodify_frame_parameters, 2, 2, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame, alist)
-     Lisp_Object frame, alist;
-{
-  Lisp_Object tail, elt, prop, val;
-  FRAME_PTR f;
-
-  if (NILP (frame))
-    f = selected_frame;
-  else
-    {
-      CHECK_LIVE_FRAME (frame, 0);
-      f = XFRAME (frame);
-    }
-
-#ifdef MSDOS
-  if (FRAME_X_P (frame))
-    IT_set_frame_parameters (XFRAME (frame), alist);
-  else
-#endif
-    for (tail = alist; !EQ (tail, Qnil); tail = Fcdr (tail))
-      {
-	elt = Fcar (tail);
-	prop = Fcar (elt);
-	val = Fcdr (elt);
-	if (EQ (prop, Qmenu_bar_lines))
-	  set_menu_bar_lines (f, val, make_number (FRAME_MENU_BAR_LINES (f)));
-      }
-
-  return Qnil;
-}
-
-DEFUN ("frame-live-p", Fframe_live_p, Sframe_live_p, 1, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  return Qt;
-}
-
-DEFUN ("frame-visible-p", Fframe_visible_p, Sframe_visible_p, 1, 1, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  (frame)
-     Lisp_Object frame;
-{
-  return Qt;
-}
-
-DEFUN ("frame-list", Fframe_list, Sframe_list, 0, 0, 0,
-  /* Don't confuse make-docfile by having two doc strings for this function.
-     make-docfile does not pay attention to #if, for good reason!  */
-  0)
-  ()
-{
-  return Fcons (Fselected_frame (), Qnil);
-}
-
-syms_of_frame ()
-{
-  syms_of_frame_1 ();
-
-  DEFVAR_LISP ("terminal-frame", &Vterminal_frame,
-  /* Don't confuse make-docfile by having two doc strings for this variable.
-     make-docfile does not pay attention to #if, for good reason!  */
-	       0);
-  XSETFASTINT (Vterminal_frame, 0);
-
-  defsubr (&Sselected_frame);
-  defsubr (&Sactive_minibuffer_window);
-  defsubr (&Swindow_frame);
-  defsubr (&Sframe_first_window);
-  defsubr (&Sframep);
-  defsubr (&Sframe_char_height);
-  defsubr (&Sframe_char_width);
-  defsubr (&Sframe_pixel_height);
-  defsubr (&Sframe_pixel_width);
-  defsubr (&Sset_frame_height);
-  defsubr (&Sset_frame_width);
-  defsubr (&Sset_frame_size);
-  defsubr (&Sset_screen_height);
-  defsubr (&Sset_screen_width);
-  defsubr (&Sframe_height);
-  Ffset (intern ("screen-height"), intern ("frame-height"));
-  defsubr (&Sframe_width);
-  Ffset (intern ("screen-width"), intern ("frame-width"));
-  defsubr (&Smouse_position);
-  Ffset (intern ("mouse-pixel-position"), intern ("mouse-position"));
-  defsubr (&Sframe_parameters);
-  defsubr (&Smodify_frame_parameters);
-  defsubr (&Sframe_live_p);
-  defsubr (&Sframe_visible_p);
-  defsubr (&Sframe_list);
-
-#ifdef MSDOS
-  /* A comment in dispnew.c says the_only_frame is not protected.  */
-  the_only_frame.face_alist = Qnil;
-  staticpro (&the_only_frame.face_alist);
-  the_only_frame.menu_bar_items = Qnil;
-  staticpro (&the_only_frame.menu_bar_items);
-  the_only_frame.menu_bar_vector = Qnil;
-  staticpro (&the_only_frame.menu_bar_vector);
-  the_only_frame.menu_bar_items = menu_bar_items (Qnil);
-#endif
-}
-
-keys_of_frame ()
-{
-}
-
-#endif /* not MULTI_FRAME */
