@@ -78,9 +78,16 @@ struct kboard
        commands that set the prefix argument.  */
     Lisp_Object Vlast_command;
 
+    /* Normally same as last-command, but never modified by
+       other commands.  */
+    Lisp_Object Vreal_last_command;
+
     /* The prefix argument for the next command, in raw form.  */
     Lisp_Object Vprefix_arg;
 
+    /* Saved prefix argument for the last command, in raw form.  */
+    Lisp_Object Vlast_prefix_arg;
+
     /* Unread events specific to this kboard.  */
     Lisp_Object kbd_queue;
 
@@ -203,6 +210,36 @@ extern int this_command_key_count;
    generated by the next character.  */
 extern Lisp_Object internal_last_event_frame;
 
+/* This holds a Lisp vector that holds the properties of a single
+   menu item while decoding it in parse_menu_item.
+   Using a Lisp vector to hold this information while we decode it
+   takes care of protecting all the data from GC.  */
+extern Lisp_Object item_properties;
+ 
+/* This describes the elements of item_properties.
+   The first element is not a property, it is a pointer to the item properties
+   that is saved for GC protection. */
+#define ITEM_PROPERTY_ITEM 0
+/* The item string.  */
+#define ITEM_PROPERTY_NAME 1
+/* Start of initilize to nil */
+/* The binding: nil, a command or a keymap.  */
+#define ITEM_PROPERTY_DEF 2
+/* The keymap if the binding is a keymap, otherwise nil.  */
+#define ITEM_PROPERTY_MAP 3
+/* Nil, :radio or :toggle.  */
+#define ITEM_PROPERTY_TYPE 4
+/* Nil or a string describing an equivalent key binding.  */
+#define ITEM_PROPERTY_KEYEQ 5
+/* Not nil if a selected toggle box or radio button, otherwise nil.  */
+#define ITEM_PROPERTY_SELECTED 6
+/* Place for a help string. Not yet used.  */
+#define ITEM_PROPERTY_HELP 7
+/* Start of initilize to t */
+/* Last property. */
+/* Not nil if item is enabled.  */
+#define ITEM_PROPERTY_ENABLE 8
+
 /* Macros for dealing with lispy events.  */
 
 /* True iff EVENT has data fields describing it (i.e. a mouse click).  */
@@ -249,11 +286,46 @@ extern Lisp_Object Qscroll_bar_movement;
 /* Symbols to use for non-text mouse positions.  */
 extern Lisp_Object Qmode_line, Qvertical_line;
 
-extern Lisp_Object get_keymap_1 ();
-extern Lisp_Object Fkeymapp ();
-extern Lisp_Object reorder_modifiers ();
-extern Lisp_Object read_char ();
+/* Forward declaration for prototypes.  */
+struct input_event;
+
+extern Lisp_Object get_keymap_1 P_ ((Lisp_Object, int, int));
+EXFUN (Fkeymapp, 1);
+extern Lisp_Object parse_modifiers P_ ((Lisp_Object));
+extern Lisp_Object reorder_modifiers P_ ((Lisp_Object));
+extern Lisp_Object read_char P_ ((int, int, Lisp_Object *, Lisp_Object, int *));
 /* User-supplied string to translate input characters through.  */
 extern Lisp_Object Vkeyboard_translate_table;
 
-extern Lisp_Object map_prompt ();
+extern Lisp_Object map_prompt P_ ((Lisp_Object));
+
+
+extern int parse_menu_item P_ ((Lisp_Object, int, int));
+
+extern void echo_now P_ ((void));
+extern void init_kboard P_ ((KBOARD *));
+extern void delete_kboard P_ ((KBOARD *));
+extern void single_kboard_state P_ ((void));
+extern void push_frame_kboard P_ ((struct frame *));
+extern void pop_frame_kboard P_ ((void));
+extern void record_asynch_buffer_change P_ ((void));
+extern void clear_waiting_for_input P_ ((void));
+extern SIGTYPE input_poll_signal P_ ((int));
+extern void start_polling P_ ((void));
+extern void stop_polling P_ ((void));
+extern void set_poll_suppress_count P_ ((int));
+extern void gobble_input P_ ((int));
+extern int input_polling_used P_ ((void));
+extern void clear_input_pending P_ ((void));
+extern int requeued_events_pending_p P_ ((void));
+extern void bind_polling_period P_ ((int));
+extern void stuff_buffered_input P_ ((Lisp_Object));
+extern void clear_waiting_for_input P_ ((void));
+extern void swallow_events P_ ((int));
+extern int help_char_p P_ ((Lisp_Object));
+extern void quit_throw_to_read_char P_ ((void));
+extern void cmd_error_internal P_ ((Lisp_Object, char *));
+extern void timer_start_idle P_ ((void));
+extern void timer_stop_idle P_ ((void));
+extern int lucid_event_type_list_p P_ ((Lisp_Object));
+extern void kbd_buffer_store_event P_ ((struct input_event *));
