@@ -22,9 +22,6 @@
 
 #undef BSD_PGRPS
 
-/* The dumped Emacs records the timezone it was dumped in.  */
-#define LOCALTIME_CACHE
-
 /* System uses OXTABS instead of the expected TAB3.
    (Copied from netbsd.h.)  */
 #define TABDLY OXTABS
@@ -32,9 +29,8 @@
 
 #define SYSV_SYSTEM_DIR
 
-/* this silences a few compilation warnings */
-#undef BSD
-#define BSD 199103
+#define HAVE_TERMIOS
+#define NO_TERMIO
 
 #define WAITTYPE int
 /* get this since it won't be included if WAITTYPE is defined */
@@ -45,3 +41,5 @@
 #ifndef WCOREDUMP
 #define WCOREDUMP(w) ((w) & 0200)
 #endif
+
+#define GETPGRP_NO_ARG 1
