@@ -28,8 +28,13 @@ Boston, MA 02111-1307, USA.  */
 #undef SYSTEM_TYPE
 #define SYSTEM_TYPE "gnu"
 
+#undef NLIST_STRUCT
+
 /* XXX should getloadavg be in libc?  Should we have a libutil?
 #define HAVE_GETLOADAVG */
+#undef NLIST_STRUCT
+#undef KERNEL_FILE
+#undef LDAV_SYMBOL
 
 #define SIGNALS_VIA_CHARACTERS
 
@@ -42,6 +47,12 @@ Boston, MA 02111-1307, USA.  */
 #define TABDLY OXTABS
 #define TAB3 OXTABS
 
+/* Tell Emacs that we are a terminfo based system; disable the use
+   of local termcap.  (GNU uses ncurses.) */
+#ifdef HAVE_LIBNCURSES
+#define TERMINFO
+#define LIBS_TERMCAP -lncurses
+#endif
 
 #define SYSV_SYSTEM_DIR
 
